@@ -47,7 +47,7 @@ in this Software without prior written authorization from The Open Group.
 #include <X11/extensions/multibufst.h>
 
 #include <stdio.h>
-#if !defined(WIN32) && !defined(Lynx)
+#if !defined(WIN32)
 #include <sys/time.h>
 #endif
 
@@ -61,8 +61,10 @@ in this Software without prior written authorization from The Open Group.
 
 static int		MultibufferEventBase;
 static int		MultibufferErrorBase;
-static DevPrivateKey MultibufferScreenPrivKey = &MultibufferScreenPrivKey;
-static DevPrivateKey MultibufferWindowPrivKey = &MultibufferWindowPrivKey;
+static int MultibufferScreenPrivKeyIndex;
+static DevPrivateKey MultibufferScreenPrivKey = &MultibufferScreenPrivKeyIndex;
+static int MultibufferWindowPrivKeyIndex;
+static DevPrivateKey MultibufferWindowPrivKey = &MultibufferWindowPrivKeyIndex;
 
 static void		PerformDisplayRequest (
 				MultibuffersPtr * /* ppMultibuffers */,
@@ -262,10 +264,10 @@ ExtensionEntry	*extEntry;
 
 static int
 ProcGetBufferVersion (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     xMbufGetBufferVersionReply	rep;
-    register int		n;
+    int		n;
 
     REQUEST_SIZE_MATCH (xMbufGetBufferVersionReq);
     rep.type = X_Reply;
@@ -421,11 +423,11 @@ CreateImageBuffers (pWin, nbuf, ids, action, hint)
 
 static int
 ProcCreateImageBuffers (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST(xMbufCreateImageBuffersReq);
     xMbufCreateImageBuffersReply	rep;
-    register int		n;
+    int		n;
     WindowPtr			pWin;
     XID				*ids;
     int				len, nbuf, i, err, rc;
@@ -486,7 +488,7 @@ ProcCreateImageBuffers (client)
 
 static int
 ProcDisplayImageBuffers (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST(xMbufDisplayImageBuffersReq);
     MultibufferPtr	    *pMultibuffer;
@@ -559,7 +561,7 @@ MultibufferResType);
 
 static int
 ProcDestroyImageBuffers (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST (xMbufDestroyImageBuffersReq);
     WindowPtr	pWin;
@@ -575,7 +577,7 @@ ProcDestroyImageBuffers (client)
 
 static int
 ProcSetMBufferAttributes (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST (xMbufSetMBufferAttributesReq);
     WindowPtr	pWin;
@@ -674,7 +676,7 @@ ProcGetMBufferAttributes (client)
 
 static int
 ProcSetBufferAttributes (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST(xMbufSetBufferAttributesReq);
     MultibufferPtr	pMultibuffer;
@@ -716,7 +718,7 @@ ProcSetBufferAttributes (client)
 
 int
 ProcGetBufferAttributes (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST(xMbufGetBufferAttributesReq);
     MultibufferPtr	pMultibuffer;
@@ -760,7 +762,7 @@ ProcGetBufferAttributes (client)
 
 static int
 ProcGetBufferInfo (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST (xMbufGetBufferInfoReq);
     DrawablePtr		    pDrawable;
@@ -825,7 +827,7 @@ ProcGetBufferInfo (client)
 
 static int
 ProcClearImageBufferArea (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST (xMbufClearImageBufferAreaReq);
     MultibufferPtr	pMultibuffer;
@@ -887,7 +889,7 @@ ProcClearImageBufferArea (client)
 
 static int
 ProcMultibufferDispatch (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST(xReq);
     switch (stuff->data) {
@@ -918,9 +920,9 @@ ProcMultibufferDispatch (client)
 
 static int
 SProcGetBufferVersion (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufGetBufferVersionReq);
 
     swaps (&stuff->length, n);
@@ -929,9 +931,9 @@ SProcGetBufferVersion (client)
 
 static int
 SProcCreateImageBuffers (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufCreateImageBuffersReq);
 
     swaps (&stuff->length, n);
@@ -943,9 +945,9 @@ SProcCreateImageBuffers (client)
 
 static int
 SProcDisplayImageBuffers (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufDisplayImageBuffersReq);
     
     swaps (&stuff->length, n);
@@ -958,9 +960,9 @@ SProcDisplayImageBuffers (client)
 
 static int
 SProcDestroyImageBuffers (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufDestroyImageBuffersReq);
     
     swaps (&stuff->length, n);
@@ -971,9 +973,9 @@ SProcDestroyImageBuffers (client)
 
 static int
 SProcSetMBufferAttributes (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufSetMBufferAttributesReq);
 
     swaps (&stuff->length, n);
@@ -986,9 +988,9 @@ SProcSetMBufferAttributes (client)
 
 static int
 SProcGetMBufferAttributes (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufGetMBufferAttributesReq);
 
     swaps (&stuff->length, n);
@@ -999,9 +1001,9 @@ SProcGetMBufferAttributes (client)
 
 static int
 SProcSetBufferAttributes (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufSetBufferAttributesReq);
 
     swaps (&stuff->length, n);
@@ -1014,9 +1016,9 @@ SProcSetBufferAttributes (client)
 
 static int
 SProcGetBufferAttributes (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufGetBufferAttributesReq);
 
     swaps (&stuff->length, n);
@@ -1027,9 +1029,9 @@ SProcGetBufferAttributes (client)
 
 static int
 SProcGetBufferInfo (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
-    register int    n;
+    int    n;
     REQUEST (xMbufGetBufferInfoReq);
 
     swaps (&stuff->length, n);
@@ -1040,9 +1042,9 @@ SProcGetBufferInfo (client)
 
 static int
 SProcClearImageBufferArea(client)
-    register ClientPtr client;
+    ClientPtr client;
 {
-    register char n;
+    char n;
     REQUEST(xMbufClearImageBufferAreaReq);
 
     swaps(&stuff->length, n);
@@ -1057,7 +1059,7 @@ SProcClearImageBufferArea(client)
 
 static int
 SProcMultibufferDispatch (client)
-    register ClientPtr	client;
+    ClientPtr	client;
 {
     REQUEST(xReq);
     switch (stuff->data) {
@@ -1277,7 +1279,7 @@ QueueDisplayRequest (client, activateTime)
     /* swap the request back so we can simply re-execute it */
     if (client->swapped)
     {
-    	register int    n;
+    	int    n;
     	REQUEST (xMbufDisplayImageBuffersReq);
     	
     	SwapRestL(stuff);
@@ -1350,9 +1352,9 @@ MultibufferExpose (pMultibuffer, pRegion)
     {
 	xEvent *pEvent;
 	PixmapPtr   pPixmap;
-	register xEvent *pe;
-	register BoxPtr pBox;
-	register int i;
+	xEvent *pe;
+	BoxPtr pBox;
+	int i;
 	int numRects;
 
 	pPixmap = pMultibuffer->pPixmap;
@@ -1662,7 +1664,7 @@ OtherClientDelete (value, id)
     XID		id;
 {
     MultibufferPtr	pMultibuffer = (MultibufferPtr)value;
-    register OtherClientsPtr	other, prev;
+    OtherClientsPtr	other, prev;
 
     prev = 0;
     for (other = pMultibuffer->otherClients; other; other = other->next)
